/*
 * Copyright (C) 1998-2003 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */
/*
 *
 * Maher Khatib - Mon Feb  2 1998
 *
 */
#ifndef _SICK_LIB_LOCAL_STR_H_
#define _SICK_LIB_LOCAL_STR_H_

/* cf. doc. SICK-PLS and SICK-LMS2xx-xxx06 */

/* STX -- start byte */
#define SICK_LIB_STX                                ((char)0x02)

/* ADR -- Adress byte */
#define SICK_LIB_ADR                                ((char)0x00)

/* ADR -- Host address */
#define STATION_ADR                             ((char)0x80)

/* Max length of a general command */
#define SICK_LIB_GEN_CMD_LENGTH_MAX                 (0x400)

/* Responses */
#define SICK_LIB_ACK                                ((char)0x06)
#define SICK_LIB_NACK                               ((char)0x15)

/* Pass word */
#define SICK_LIB_PASSWD "SICK_LMS"

/* genertator check sum poly -- x15 + x2 + 1 */
#define CRC16_GEN_POLY ((unsigned short)0x8005)

/* Response timeout for control tram */
#define SICK_LIB_CONTROL_TRAM_TIMEOUT               (20)

/* INTEL short format */
#ifndef __i386
#define SICK_LIB_SHORT_FORMAT(x) ((unsigned short) (((x) << 8) | (((x) >> 8) & 0x00ff)))
#else
#define SICK_LIB_SHORT_FORMAT(x) (x)
#endif

/* specials tram lengths */
#define SICK_LIB_CONTROL_TRAM_LENGTH (7)
#define SICK_LIB_MODE_TRAM_LENGTH (8)

/* serial package defintions */
#define SICK_LIB_SERIAL_IN_BUFFER_LENGTH      (0x2000)
#define SICK_LIB_SERIAL_OUT_BUFFER_LENGTH     (0x800)
#define SICK_LIB_SERIAL_READ_TASK_PRIORITY    (5)
#define SICK_LIB_SERIAL_WRITE_TASK_PRIORITY   (10)

/*
 * Control tram default
 */
#define SICK_LIB_CONTROL_TRAM_DEFAULT {SICK_LIB_STX, SICK_LIB_ADR, 0, 0, 0, 0}
#define SICK_LIB_MODE_TRAM_DEFAULT {SICK_LIB_STX, SICK_LIB_ADR, 0, 0, 0, 0}

/*
 * Possible tranisitions
 */
#include "sick_lib_transition.h"

/*
 * User mode tram
 */
typedef struct sick_lib_mode_tram_str {
  unsigned int stx:8,
    adrs:8,
    len:16;
  unsigned int cmd:8,
    mode:8,
    crc:16;
} SICK_LIB_MODE_TRAM_STR;

/*
 * User control tram
 */
typedef struct sick_lib_control_tram_str {
  unsigned int stx:8,
    adrs:8,
    len:16;
  unsigned int cmd:8,
    crc:16,
    unused:8;
} SICK_LIB_CONTROL_TRAM_STR;

/* local types */
typedef char TYPE_7[7];
typedef char TYPE_9[9];

/*
 * Sick state str
 */
typedef struct sick_lib_internal_status_str {
  TYPE_7 version;
  unsigned char mode;
  unsigned char status;
  TYPE_9 code;
  unsigned char variant;		    /* VOLATILE / NON_VOLATILE */

  unsigned short pollution_value;	    /* value */
  unsigned short pollution_ref;

  unsigned short pollution_channel_calib;
  unsigned short ref_pollution_channel_calib;

  unsigned short motor_speed;
  unsigned short dummy1;

  unsigned short signal_100_dark_1;
  unsigned short dummy2;

  unsigned short signal_100_dark_2;
  unsigned short signal_66_dark_1;

  unsigned short dummy3;
  unsigned short signal_66_dark_2;

  unsigned short dummy4;
  unsigned short laser_power;

  unsigned short peak_threshold;
  unsigned short power_measure_angle;

  unsigned short signal_calib_value;
  unsigned short stop_threshold_expected_value;

  unsigned short peak_threshold_expected_value;
  unsigned short stop_threshold_value;

  unsigned short peak_threshold_value;
  unsigned short dummy9;

  unsigned short measure_setting;
  unsigned short dummy10;

  unsigned short dummy11;
  unsigned short scan_angle;

  unsigned short resolution;
  unsigned char restart_mode;
  unsigned short restart_time;			    /* in 5ms */
  unsigned char restart_test;
  unsigned char out_switchs_def;
  unsigned short baud;
  unsigned char multiple_eval;		    /* [2 - 125] */
  unsigned char permanent_baud;
  unsigned char adrs;

  unsigned char field_set;
  unsigned char  measure_unit;
  unsigned char laser_state;
  TYPE_7 eprom_version;
  unsigned char rest[100];
} SICK_LIB_INTERNAL_STATUS_STR;

/*
 * Sick scan str
 */
typedef struct sick_lib_scan_str {
  unsigned char status;			    /* 0 = ABORT - 1 = OK */
  unsigned short range;			    /* 100 or 180 deg */
  unsigned short resolution;		    /* [100, 50, 25] / 100 deg */
  unsigned char rest[7];
} SICK_LIB_SCAN_STR;

#endif /* _SICK_LIB_LOCAL_STR_H_ */
